# frozen_string_literal: true

module Store
  module Admin
    class ProductsController < AdminController
      before_action :find_products, only: %i[index more sort]
      before_action :find_product, only: %i[admin_edit admin_update promote unpromote publish unpublish]

      def index
        authorize Product

        respond_to do |format|
          format.html
          format.turbo_stream do
            render turbo_stream: [
              turbo_stream.update(:app_content, render_to_string(template: 'store/admin/products/index', layout: false, formats: [:html])),
              turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages'),
              turbo_stream.push_state(request.url)
            ]
          end
        end
      end

      def more
        authorize Product

        respond_to(&:turbo_stream)
      end

      def sort
        authorize Product

        render partial: 'store/admin/products/table'
      end

      def admin_edit
        authorize @product

        respond_to(&:html)
      end

      def admin_update
        authorize @product

        if @product.update(product_params)
          flash[:notice] = t('store.admin.product.updated_successfully')
          redirect_to store_admin_products_path, status: :see_other
        else
          render turbo_stream: [
            turbo_stream.update(:app_content, render_to_string(template: 'store/admin/products/admin_edit', layout: false, formats: [:html]))
          ], status: :unprocessable_entity
        end
      end

      def promote
        authorize @product

        @product.update(promoted: true)

        flash.now[:notice] = t('store.admin.product.promoted_successfully')
        render turbo_stream: [
          turbo_stream.replace(@product, partial: 'store/admin/products/row', locals: { model: @product }),
          turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
        ]
      end

      def unpromote
        authorize @product

        @product.update(promoted: false)

        flash.now[:notice] = t('store.admin.product.unpromoted_successfully')
        render turbo_stream: [
          turbo_stream.replace(@product, partial: 'store/admin/products/row', locals: { model: @product }),
          turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
        ]
      end

      def publish
        authorize @product

        @product.publish

        flash.now[:notice] = t('store.admin.product.published_successfully')
        render turbo_stream: [
          turbo_stream.replace(@product, partial: 'store/admin/products/row', locals: { model: @product }),
          turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
        ]
      end

      def unpublish
        authorize @product

        @product.unpublish

        flash.now[:notice] = t('store.admin.product.unpublished_successfully')
        render turbo_stream: [
          turbo_stream.replace(@product, partial: 'store/admin/products/row', locals: { model: @product }),
          turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
        ]
      end

      private

      def find_products
        controller_scope = {}
        if (@search = params[:search])
          regex = Regexp.new(Regexp.escape(@search), Regexp::IGNORECASE)
          controller_scope.merge!(title: /#{regex}/i)
        end
        @models = find_models(policy_scope(Product).where(controller_scope), default_sort_by: 'title')
      end

      def find_product
        @product = policy_scope(Product).find(params[:id])
      end

      def product_params
        cleaned_params = params.require(:product).permit(
          :title,
          :description,
          :price,
          :duration_count,
          :duration_unit,
          :vat,
          :free,
          picture_ids: []
        )
        cleaned_params[:picture_ids] = [] if cleaned_params[:picture_ids].nil?

        cleaned_params
      end
    end
  end
end
