# frozen_string_literal: true

module Store
  module Admin
    class UsersController < AdminController
      before_action :find_users, only: %i[index more sort]
      before_action :find_user, only: %i[edit update block unblock]

      def index
        authorize User, policy_class: Store::UserPolicy

        respond_to do |format|
          format.html
          format.turbo_stream do
            render turbo_stream: [
              turbo_stream.update(:app_content, render_to_string(template: 'store/admin/users/index', layout: false, formats: [:html])),
              turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages'),
              turbo_stream.push_state(request.url)
            ]
          end
        end
      end

      def more
        authorize User, policy_class: Store::UserPolicy

        respond_to(&:turbo_stream)
      end

      def sort
        authorize User, policy_class: Store::UserPolicy

        render partial: 'store/admin/users/table'
      end

      def edit
        authorize @user, policy_class: Store::UserPolicy

        respond_to(&:html)
      end

      def update
        authorize @user, policy_class: Store::UserPolicy

        if @user.update(user_params)
          flash[:notice] = t('store.admin.user.updated_successfully')
          redirect_to store_admin_users_path, status: :see_other
        else
          render turbo_stream: [
            turbo_stream.update(:app_content, render_to_string(template: 'store/admin/users/edit', layout: false, formats: [:html]))
          ], status: :unprocessable_entity
        end
      end

      def block
        authorize @user, policy_class: Store::UserPolicy

        @user.block

        flash[:notice] = t('store.admin.user.blocked_successfully')
        redirect_to store_admin_users_path
      end

      def unblock
        authorize @user, policy_class: Store::UserPolicy

        @user.unblock

        flash[:notice] = t('store.admin.user.unblocked_successfully')
        redirect_to store_admin_users_path
      end

      private

      def find_users
        controller_scope = {}
        if (@search = params[:search])
          search_term = Regexp.escape(@search)
          regex = Regexp.new(search_term, Regexp::IGNORECASE)
          controller_scope.merge!(cleaned_main_key: /#{regex}/i)
        end
        @models = find_models(policy_scope(User, policy_scope_class: Store::UserPolicy::Scope).where(controller_scope), default_sort_by: 'email')
      end

      def find_user
        @user = policy_scope(User, policy_scope_class: Store::UserPolicy::Scope).find(params[:id])
      end

      def user_params
        cleaned_params = params.require(:user).permit(:email, :login, :first_name, :last_name, :position)

        if current_user.admin?
          roles = @user&.auth_roles || []

          if params[:user][:admin] == 'on'
            roles |= ['Administrator']
          else
            roles -= ['Administrator']
          end

          cleaned_params[:roles] = roles
        end

        cleaned_params
      end
    end
  end
end
