# frozen_string_literal: true

module Store
  class ApplicationController < ActionController::Base
    protect_from_forgery with: :exception
    before_action :try_to_set_user_from_token
    before_action :authenticate_user!
    rescue_from StandardError, with: :error_server
    rescue_from Pundit::NotAuthorizedError, with: :user_not_authorized

    include Store::SetLocale
    include Pundit::Authorization

    after_action :verify_authorized
    after_action :verify_policy_scoped

    def authenticate_user!
      return redirect_to(new_store_user_session_path, alert: 'You need to sign up before continuing.') unless user_signed_in?

      @user = current_user
    end

    def auth_user_token
      request.headers['X-auth-user-token']
    end

    private

    def hide_sidebar
      @hide_sidebar = true
    end

    def try_to_set_user_from_token
      return unless auth_user_token

      user = User.find_by(authentication_token: auth_user_token)

      return unless user && user != warden.user

      warden.set_user(user)
    end

    def check_if_developer
      return if @user&.developer?

      flash.now[:alert] = 'You are not a developer'
      redirect_to(store_home_path)
    end

    def check_if_customer
      return if @user&.customer?

      flash.now[:alert] = 'You are not a customer'
      redirect_to(store_home_path)
    end

    def find_models(scope, options = {})
      @sort_by = params.fetch(:sort_by, options[:default_sort_by] || 'title').to_s
      @sort_direction = params.fetch(:sort_direction, 'asc').to_s
      @page = params.fetch(:page, 1).to_i

      per_page = options[:per_page] || 15

      models = scope.order_by(@sort_by => @sort_direction).skip((@page - 1) * per_page).limit(per_page)
      @models_count = models.count

      total_pages = (@models_count / per_page.to_f).ceil
      @next_page = total_pages > @page
      @total_count = scope.count

      models
    end

    def error_server
      skip_authorization
      skip_policy_scope

      @hide_sidebar = true unless @user
      respond_to do |format|
        format.html do
          render 'store/errors/error_server', status: :ok
        end
        format.turbo_stream do
          flash.now[:alert] = t('store.errors.500_notice')
          render turbo_stream: [
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
          ], status: :internal_server_error
        end
      end
    end

    def user_not_authorized
      skip_authorization
      skip_policy_scope

      @hide_sidebar = true unless @user
      respond_to do |format|
        format.html do
          render 'store/errors/user_not_authorized', status: :ok
        end
        format.turbo_stream do
          flash.now[:alert] = t('store.errors.403_notice')
          render turbo_stream: [
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
          ], status: :internal_server_error
        end
      end
    end
  end
end
