# frozen_string_literal: true

module Store
  class AppsController < ApplicationController
    before_action :check_if_developer
    before_action :find_apps, only: %i[index more sort]
    before_action :find_app, only: %i[edit update new_product create_product]

    def index
      authorize App, policy_class: Store::AppPolicy

      respond_to do |format|
        format.html
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.update(:app_content, render_to_string(template: 'store/apps/index', layout: false, formats: [:html])),
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages'),
            turbo_stream.push_state(request.url)
          ]
        end
      end
    end

    def more
      authorize App, policy_class: Store::AppPolicy

      respond_to(&:turbo_stream)
    end

    def sort
      authorize App, policy_class: Store::AppPolicy

      render partial: 'store/apps/apps_table'
    end

    def edit
      authorize @app, policy_class: Store::AppPolicy

      respond_to(&:html)
    end

    def update
      authorize @app, policy_class: Store::AppPolicy

      if @app.update(app_params)
        flash[:notice] = t('store.app.updated_successfully')
        redirect_to store_apps_path, status: :see_other
      else
        render turbo_stream: [
          turbo_stream.update(:app_content, render_to_string(template: 'store/apps/edit', layout: false, formats: [:html]))
        ], status: :unprocessable_entity
      end
    end

    def new
      authorize App, policy_class: Store::AppPolicy

      skip_policy_scope
      @app = App.new

      respond_to(&:html)
    end

    def create
      authorize App, policy_class: Store::AppPolicy
      skip_policy_scope

      @app = App.new(app_params)
      @app.user = @user

      if @app.save
        flash[:notice] = t('store.app.created_successfully')

        redirect_to store_apps_path, status: :see_other
      else
        render turbo_stream: [
          turbo_stream.update(:app_content, render_to_string(template: 'store/apps/new', layout: false, formats: [:html]))
        ], status: :unprocessable_entity
      end
    end

    def new_product
      authorize @app, policy_class: Store::AppPolicy

      @product = @app.to_product

      respond_to(&:html)
    end

    def create_product
      authorize @app, policy_class: Store::AppPolicy

      @product = @app.to_product
      @product.user = @user

      if @product.update(product_params)
        flash[:notice] = t('store.app.product_created_successfully')

        redirect_to store_apps_path, status: :see_other
      else
        render turbo_stream: [
          turbo_stream.update(:app_content, render_to_string(template: 'store/apps/new_product', layout: false, formats: [:html]))
        ], status: :unprocessable_entity
      end
    end

    private

    def find_apps
      @apps = find_models(policy_scope(App, policy_scope_class: Store::AppPolicy::Scope), default_sort_by: 'name')
    end

    def find_app
      @app = policy_scope(App, policy_scope_class: Store::AppPolicy::Scope).find(params[:id])
    end

    def app_params
      params.require(:app).permit(:name, :package, :theme_color, :icon)
    end

    def product_params
      params.require(:product).permit(
        :title,
        :description,
        :price,
        :duration_count,
        :duration_unit,
        :vat,
        :free
      )
    end
  end
end
