# frozen_string_literal: true

module Store
  class HomeController < ApplicationController
    skip_before_action :authenticate_user!
    before_action :check_user
    before_action :hide_sidebar
    before_action :find_products, only: %i[product_details more_reviews create_review destroy_review]
    before_action :find_user_review, only: %i[product_details more_reviews destroy_review]
    before_action :find_other_reviews, only: %i[product_details more_reviews]

    def index
      skip_authorization
      per_page = 20
      @carrousel_products = policy_scope(Product).published.where(promoted: true)

      @page = params[:page]&.to_i || 1
      @products = policy_scope(Product).published.skip((@page - 1) * per_page).limit(per_page)
      total_pages = (@products.count / per_page.to_f).ceil
      @next_page = total_pages > @page

      respond_to do |format|
        format.html
        format.turbo_stream
      end
    end

    def search
      skip_authorization

      per_page = 6

      @search = params[:search]
      @products = policy_scope(Product).published
      if @search.present?
        search_term = Regexp.escape(@search)
        regex = Regexp.new(search_term, Regexp::IGNORECASE)
        @products = @products.where(title: /#{regex}/i)
      end

      @page = params[:page]&.to_i || 1
      @products = @products.skip((@page - 1) * per_page).limit(per_page)
      total_pages = (@products.count / per_page.to_f).ceil
      @next_page = total_pages > @page

      @all_product_count = policy_scope(Product).published.count
      @result_product_count = @products.count

      respond_to do |format|
        format.html
        format.turbo_stream
      end
    end

    def product_details
      authorize @product
      policy_scope(Product)

      @review = Review.new

      respond_to do |format|
        format.html
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.replace(:review_part, partial: 'store/home/review_part'),
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
          ]
        end
      end
    end

    def more_reviews
      skip_authorization
      respond_to(&:turbo_stream)
    end

    def create_review
      authorize Review
      @review = Review.new(review_params)
      @review.user_id = @user.id
      @review.product_id = @product.id

      if @review.save
        flash[:notice] = t('store.product_details.review_created_successfully')
        redirect_to product_details_store_product_path(@product), status: :see_other
      else
        render partial: 'store/home/review_form', status: :unprocessable_entity
      end
    end

    def destroy_review
      authorize @user_review
      @user_review.destroy

      flash[:notice] = t('store.product_details.review_deleted_successfully')
      redirect_to product_details_store_product_path(@product), status: :see_other
    end

    def update_locale
      skip_authorization
      skip_policy_scope
      return unless (locale = params[:locale].to_s.strip.to_sym)

      update_cookie_locale(locale)
      I18n.locale = locale

      render turbo_stream: [
        turbo_stream.reload
      ]
    end

    private

    def check_user
      @user = current_user
    end

    def review_params
      params.require(:review).permit(:comment, :rating)
    end

    def find_products
      @product = policy_scope(Product).find(params[:id])
    end

    def find_user_review
      @user_review = @user && policy_scope(Review).find_by(product_id: @product.id, user_id: @user.id)
    end

    def find_other_reviews
      @other_reviews = find_models(policy_scope(Review).where(product_id: @product&.id, :id.ne => @user_review&.id), per_page: 5)
    end
  end
end
