# frozen_string_literal: true

module Store
  class OrdersController < ApplicationController
    before_action :check_if_customer
    before_action :find_user_cart, only: %i[buy cart remove_from_cart checkout add_to_cart]

    def buy
      authorize @order

      @order.add_to_cart!(policy_scope(Product).find(params[:product_id]), @user)

      redirect_to cart_store_orders_path
    end

    def add_to_cart
      authorize @order

      @order.add_to_cart!(policy_scope(Product).find(params[:product_id]), @user)

      flash[:notice] = t('store.cart.product_added_to_your_cart')

      respond_to do |format|
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.update('cart-link', partial: 'store/shared/cart_link'),
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
          ]
        end
      end
    end

    def cart
      authorize @order
      skip_policy_scope

      @possible_hosts = %w[localhost rec.plugandwork.com]

      respond_to do |format|
        format.html
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.update(:app_content, render_to_string(template: 'store/orders/cart', layout: false, formats: [:html])),
            turbo_stream.update('cart-link', partial: 'store/shared/cart_link'),
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages'),
            turbo_stream.push_state(request.url)
          ]
        end
      end
    end

    def remove_from_cart
      authorize @order
      skip_policy_scope

      @voucher = policy_scope(Voucher).find(params[:voucher_id])
      @order.vouchers.delete(@voucher)
      @order.save!

      redirect_to cart_store_orders_path
    end

    def checkout
      authorize @order
      skip_policy_scope

      if @order.update(order_params)
        flash[:notice] = t('store.cart.order_terminated_successfully')
        @order.update(payment_status: 'paid', is_cart: false)
        redirect_to store_vouchers_path
      else
        render :cart
      end
    end

    def index
      authorize Order
      @orders = find_models(policy_scope(Order).where(user: @user, is_cart: false), default_sort_by: 'created_at')
    end

    def more
      authorize Order
      @orders = find_models(policy_scope(Order).where(user: @user, is_cart: false), default_sort_by: 'created_at')
      respond_to(&:turbo_stream)
    end

    def show
      @order = policy_scope(Order).find_by(user: @user, id: params[:id])
      authorize @order
    end

    private

    def order_params
      params.require(:order).permit(vouchers_attributes: %i[id host])
    end

    def find_user_cart
      @order = @user.user_cart
    end
  end
end
