# frozen_string_literal: true

module Store
  class ProductsController < ApplicationController
    before_action :check_if_developer

    def published
      authorize Product
      @products = find_models(policy_scope(Product).published.where(user: @user))

      respond_to do |format|
        format.html
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.update(:app_content, render_to_string(template: 'store/products/published', layout: false, formats: [:html])),
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages'),
            turbo_stream.push_state(request.url)
          ]
        end
      end
    end

    def more_published
      authorize Product
      @products = find_models(policy_scope(Product).published.where(user: @user))
      respond_to(&:turbo_stream)
    end

    def sort_published
      authorize Product
      @products = find_models(policy_scope(Product).published.where(user: @user))
      render partial: 'store/products/published_table'
    end

    def unpublished
      authorize Product
      @products = find_models(policy_scope(Product).unpublished.where(user: @user))

      respond_to do |format|
        format.html
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.update(:app_content, render_to_string(template: 'store/products/unpublished', layout: false, formats: [:html])),
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages'),
            turbo_stream.push_state(request.url)
          ]
        end
      end
    end

    def sort_unpublished
      authorize Product
      @products = find_models(policy_scope(Product).unpublished.where(user: @user))
      render partial: 'store/products/unpublished_table'
    end

    def more_unpublished
      authorize Product
      @products = find_models(policy_scope(Product).unpublished.where(user: @user))
      respond_to(&:turbo_stream)
    end

    def edit
      @product = policy_scope(Product).find(params[:id])
      authorize @product

      respond_to(&:html)
    end

    def update
      @product = policy_scope(Product).find(params[:id])
      authorize @product

      if @product.update(product_params)
        flash[:notice] = t('store.product.updated_successfully')
        redirect_to unpublished_store_products_path, status: :see_other
      else
        render turbo_stream: [
          turbo_stream.update(:app_content, render_to_string(template: 'store/products/edit', layout: false, formats: [:html]))
        ], status: :unprocessable_entity
      end
    end

    def unpublish
      @product = policy_scope(Product).find(params[:id])
      authorize @product

      @product.unpublish
      flash[:notice] = t('store.product.unpublished_successfully')

      @products = find_models(policy_scope(Product).published.where(user: @user))

      respond_to do |format|
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.update(:model_table, partial: 'store/products/published_table'),
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
          ]
        end
      end
    end

    def ask_to_publish
      @product = policy_scope(Product).find(params[:id])
      authorize @product

      @product.ask_to_publish

      flash[:notice] = t('store.product.submited_for_review')
      @products = find_models(policy_scope(Product).unpublished.where(user: @user))

      respond_to do |format|
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.update(:model_table, partial: 'store/products/unpublished_table'),
            turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
          ]
        end
      end
    end

    def add_picture
      skip_authorization
      skip_policy_scope

      name = params[:asset][:file]&.original_filename

      if (@asset = Asset.create(asset_params.merge(name:)))
        render turbo_stream: [
          turbo_stream.append(:pictures, partial: 'store/products/form_picture', locals: { picture_id: @asset.id, picture_url: Asset.public_relative_url(@asset.id) })
        ], status: :created
      else
        flash[:notice] = t('store.product.picture_added_error')

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'store/shared/flash_messages')
        ], status: :unprocessable_entity
      end
    end

    private

    def product_params
      cleaned_params = params.require(:product).permit(
        :title,
        :description,
        :price,
        :duration_count,
        :duration_unit,
        :vat,
        :free,
        picture_ids: []
      )
      cleaned_params[:picture_ids] = [] if cleaned_params[:picture_ids].nil?

      cleaned_params
    end

    def asset_params
      params.require(:asset).permit(:file)
    end
  end
end
