# frozen_string_literal: true

module Store
  class Order
    include Mongoid::Document
    include Mongoid::Timestamps

    field :browser_ip, type: String
    field :currency, type: String, default: 'EUR'
    field :vat_price, type: Float, default: 0.0
    field :subtotal_price, type: Float, default: 0.0
    field :total_price, type: Float, default: 0.0
    field :active, type: Mongoid::Boolean, default: false
    field :is_cart, type: Mongoid::Boolean, default: false

    field :payment_amount, type: Float, default: 0.0
    field :payment_method, type: String
    field :payment_status, type: String, default: 'not_processed'
    field :voucher_ids, type: Array, default: []
    field :voucher_count, type: Integer, default: 0

    belongs_to :user, optional: true
    belongs_to :customer, class_name: 'Store::Customer', optional: true

    before_save :calc_price, :update_voucher_count
    after_save :gen_vouchers

    has_many :vouchers, class_name: 'Store::Voucher', inverse_of: :order
    accepts_nested_attributes_for :vouchers, allow_destroy: true

    include ::AclProtected

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    def calc_price
      self.subtotal_price = vouchers.sum do |voucher|
        voucher.product.price
      end

      self.vat_price = vouchers.sum do |voucher|
        (voucher.product.price * voucher.product.vat / 100).round(2)
      end

      self.subtotal_price = subtotal_price.round(2)
      self.vat_price = vat_price.round(2)

      self.total_price = subtotal_price + vat_price
    end

    def add_to_cart!(product, user)
      vouchers << Voucher.create(product:, user:)
      self.voucher_count = voucher_ids.length
      save!
    end

    def update_voucher_count
      return unless will_save_change_to_attribute?(:voucher_ids)

      self.voucher_count = voucher_ids.length
    end

    def gen_vouchers
      return unless payment_status == 'paid' && saved_change_to_attribute(:payment_status)

      vouchers.each do |voucher|
        voucher.gen_code
        voucher.save
      end
    end
  end
end
