# frozen_string_literal: true

module Store
  class Product
    include Mongoid::Document
    include Mongoid::Timestamps

    include ::AclProtected
    include ::Datum
    include ::Slugable

    field :title, type: String
    field :description, type: String
    field :type, type: String, default: 'app' # used for app
    field :package, type: String # to generate voucher license exemple : 'emails'
    field :labels, type: String
    field :superfields, type: Hash
    field :picture_ids, type: Array, default: []
    field :picture_urls, type: Array, default: []
    field :values, type: Hash, default: {}
    # values contains number of seats, number of users or other product variations to generate license values
    field :duration_count, type: Integer, default: 3
    field :duration_unit, type: String, default: 'day'
    field :waiting_for_approval, type: Boolean, default: false
    field :published_at, type: DateTime, default: nil
    field :promoted, type: Mongoid::Boolean, default: false
    field :free, type: Mongoid::Boolean, default: true

    field :price, type: Float
    field :vat, type: Float, default: 20.0
    field :rating, type: Float, default: 0.0
    field :rating_count, type: Integer, default: 0
    field :rating_details, type: Hash, default: { 1 => 0, 2 => 0, 3 => 0, 4 => 0, 5 => 0 }.stringify_keys

    belongs_to :user, optional: true

    validates_presence_of :price, :title, :type, :package, :duration_count, :duration_unit

    index({ title: 1 }, { background: 1 })

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    sluggify :title

    scope :published, -> { where(:published_at.ne => nil) }
    scope :unpublished, -> { where(published_at: nil) }
    scope :waiting_for_approval, -> { where(waiting_for_approval: true) }

    before_save :remove_price_if_free, :update_picture_urls
    after_save :udpate_app_product_nb, :delete_old_pictures
    after_create :force_udpate_app_product_nb

    def self.possible_duration_units
      %w[day week month year]
    end

    def main_picture_url
      picture_urls&.first
    end

    def pictures
      unordered_pictures = Asset.where(:id.in => picture_ids)
      unordered_pictures.sort_by { |picture| picture_ids.index(picture.id.to_s) }
    end

    def update_picture_urls
      return unless will_save_change_to_attribute?(:picture_ids)

      self.picture_ids = picture_ids.to_array.reject(&:blank?).map(&:to_s)
      self.picture_urls = pictures.map { |id| Asset.public_relative_url(id) }
    end

    def delete_old_pictures
      return unless saved_change_to_attribute?(:picture_ids)

      to_delete_picture_ids = saved_change_to_attribute(:picture_ids).first.to_array - picture_ids.to_array
      Asset.where(id: { :$in => to_delete_picture_ids }).each(&:destroy)
    end

    def remove_price_if_free
      return unless free

      self.price = 0
    end

    def total_price
      (price + (price * vat / 100)).round(2)
    end

    # Store::Product.create(duration_count: 1, duration_unit: 'year', package: 'contracts', title: '1 year contracts app')
    def gen_code_for(host)
      expiration = Date.today + duration_count.send(duration_unit.to_s)
      values = self.values
      l = License.create(type:, module: package, expire_at: expiration, env: Rails.env) # , active: true)
      l.token = SecureRandom.uuid
      values['app_host'] = host
      l.values = values.stringify_keys
      l.save
      l.key = Base64.encode64(OpenSSL::HMAC.digest(OpenSSL::Digest.new('sha256'), l.token, "#{l.expire_at}#{l.values}")).strip
      l.save
      code = l.reload.to_code
      l.destroy
      code
    end

    def app
      return unless type == 'app'

      @app ||= App.find_by(package:)
    end

    def update_rating
      self.rating_details = {}
      (1..5).each do |i|
        rating_details[i] = Store::Review.where(product_id: id, rating: i).count
      end
      self.rating_count = Store::Review.where(product_id: id).count
      self.rating = if rating_count.zero?
                      0
                    else
                      (rating_details.map { |k, v| k * v }.sum.to_f / rating_count).round(2)
                    end
    end

    def publish
      update(waiting_for_approval: false, published_at: Time.now)
    end

    def unpublish
      update(waiting_for_approval: false, published_at: nil)
    end

    def ask_to_publish
      update(waiting_for_approval: true)
    end

    def force_udpate_app_product_nb
      return unless app

      app.update_product_nb
      app.save
    end

    def udpate_app_product_nb
      return unless app
      return unless saved_change_to_published_at?

      app.update_product_nb
      app.save!
    end

    def published?
      published_at.present?
    end
  end
end
