# frozen_string_literal: true

module Store
  class Review
    include Mongoid::Document
    include Mongoid::Timestamps

    field :comment, type: String
    field :rating, type: Integer, default: 1
    belongs_to :user, index: true
    belongs_to :product, index: true

    index({ user_id: 1, product_id: 1 }, { unique: true, background: true })
    index({ product_id: 1, rating: 1 }, { background: true })

    validates_presence_of :user_id, :product_id
    validates_format_of :rating, with: /\A[1-5]\z/
    validate :user_can_only_review_product_once

    after_save :update_product_rating
    after_destroy :update_product_rating_after_destroy

    private

    def user_can_only_review_product_once
      existing_review = Store::Review.where(user_id:, product_id:).exists?
      errors.add(:base, 'You have already reviewed this product') if existing_review && new_record?
    end

    def update_product_rating
      return unless product && saved_change_to_attribute(:rating)

      product.update_rating
      product.save
    end

    def update_product_rating_after_destroy
      product&.update_rating
      product&.save
    end
  end
end
