# frozen_string_literal: true

require 'factory_bot_rails'
require 'database_cleaner/mongoid'

include FactoryBot::Syntax::Methods # rubocop:disable Style/MixinUsage
DatabaseCleaner.clean

FactoryBot.definition_file_paths << File.join(Rails.root, 'packages', 'store', 'spec', 'factories')
FactoryBot.factories.clear
FactoryBot.find_definitions

password = 'Password1!'

admin = create(:user, roles: ['Administrator'], email: 'admin@email.com', password:)
main_customer = create(:user, email: 'customer@email.com', position: 'customer', password:)
customers = []
customers << main_customer
30.times do |i|
  customers << create(:user, email: "customer#{i + 2}@email.com", position: 'customer', password:)
end

developers = []
main_developer = create(:user, email: 'developer@email.com', position: 'developer', password:)
developers << main_developer
10.times do |i|
  developers << create(:user, email: "developer#{i + 2}@email.com", position: 'developer', password:)
end

def create_app(user:, max_product: 3, max_reviews: 10)
  app = create(:store_app, user:)

  [1, 2, max_product].sample.times do
    product = app.to_product(rand(1..100))
    product.user_id = user.id
    3.times do
      file = Dir.glob('packages/store/app/assets/images/store/product_picture_url_samples/*').sample
      product.picture_ids << Asset.create(name: File.basename(file), file:).id
    end
    product.free = [true, false].sample
    product.save

    review_users = User.where(position: 'customer').to_a.sample([0, 3, max_reviews / 2, max_reviews].sample)
    review_users.each do |u|
      create(:store_review, product:, user: u, rating: [1, 2, 3, 4, 5].sample, comment: Faker::Lorem.paragraphs(number: 2).join("\n\n"))
    end
  end
end

15.times do
  create_app(user: main_developer, max_product: 1, max_reviews: 10)
end

developers[1..8].each do |developer|
  create_app(user: developer, max_product: 1, max_reviews: 10)
end

all_products = Store::Product.all.to_a
main_developer_products = all_products.select { |p| p.user_id == main_developer.id }

published_products = all_products.sample(all_products.length / 2) + main_developer_products.sample(2)
published_products.each do |product|
  product.reload.update(published_at: Faker::Time.between(from: 30.days.ago, to: Time.now))
end

published_products.sample(3).each do |product|
  product.update(promoted: true)
end

3.times do
  order = create(:order, user: main_customer)
  published_products.sample([1, 2, 3, 5].sample).map do |product|
    order.add_to_cart!(product, main_customer)
  end
  order.update(payment_status: 'paid', is_cart: false)
  Store::Voucher.where(order_id: order.id).each do |voucher|
    voucher.update(host: ['localhost', 'rec.plugandwork.com'].sample)
  end
end

puts "Rails environment seeded with the following data: #{Rails.env}"
puts "Users: #{User.count}"
puts admin.email
puts "Apps: #{App.count}"
puts "Products: #{Store::Product.count}"
puts "Reviews: #{Store::Review.count}"
puts "Orders: #{Store::Order.count}"
puts "Vouchers: #{Store::Voucher.count}"
