# frozen_string_literal: true

require 'rails_helper'

describe Store::OrderPolicy do
  subject { described_class }

  permissions :index?, :more?, :sort? do
    it 'denies access if not customer' do
      expect(subject).not_to permit(FactoryBot.create(:user), Store::Order)
    end

    it 'grants access if customer' do
      expect(subject).to permit(FactoryBot.create(:user, position: 'customer'), Store::Order)
    end

    it 'grants access if admin' do
      expect(subject).to permit(FactoryBot.create(:user, roles: ['Administrator']), Store::Order)
    end
  end

  permissions :show? do
    let!(:customer) { FactoryBot.create(:user, position: 'customer') }
    let!(:order) { FactoryBot.create(:order, user_id: customer.id) }

    it 'denies access if not customer or admin' do
      expect(subject).not_to permit(FactoryBot.create(:user), order)
    end

    it 'denies access if customer but not owner' do
      expect(subject).not_to permit(FactoryBot.create(:user, position: 'customer'), order)
    end

    it 'grants access if customer and owner' do
      expect(subject).to permit(customer, order)
    end

    it 'grants access if admin' do
      expect(subject).to permit(FactoryBot.create(:user, roles: ['Administrator']), order)
    end
  end

  permissions :checkout?, :remove_from_cart?, :cart?, :buy?, :add_to_cart? do
    let!(:customer) { FactoryBot.create(:user, position: 'customer') }
    let!(:order) { FactoryBot.create(:order, user_id: customer.id, is_cart: true) }

    it 'denies access if not customer or admin' do
      expect(subject).not_to permit(FactoryBot.create(:user), order)
    end

    it 'denies access if customer but not owner' do
      expect(subject).not_to permit(FactoryBot.create(:user, position: 'customer'), order)
    end

    it 'denies access if admin' do
      expect(subject).not_to permit(FactoryBot.create(:user, roles: ['Administrator']), order)
    end

    it 'denies access if customer and owner but not cart' do
      order.update(is_cart: false)
      expect(subject).not_to permit(customer, order)
    end

    it 'grants access if customer and owner' do
      expect(subject).to permit(customer, order)
    end
  end
end
