# frozen_string_literal: true

require 'rails_helper'

describe Store::ProductPolicy do
  subject { described_class }

  permissions :published?, :more_published?, :sort_published?, :unpublished?, :more_unpublished?, :sort_unpublished? do
    it 'denies access if not developer' do
      expect(subject).not_to permit(FactoryBot.create(:user), Store::Order)
    end

    it 'grants access if developer' do
      expect(subject).to permit(FactoryBot.create(:user, position: 'developer'), Store::Order)
    end

    it 'grants access if admin' do
      expect(subject).to permit(FactoryBot.create(:user, roles: ['Administrator']), Store::Order)
    end
  end

  permissions :product_details? do
    let!(:developer) { FactoryBot.create(:user, position: 'developer') }
    let!(:product) { FactoryBot.create(:product, user_id: developer.id) }

    it 'grants access if published' do
      product.update(published_at: Time.zone.now - 1.day)
      expect(subject).to permit(FactoryBot.create(:user), product)
    end

    it 'denies access if unpublished' do
      expect(subject).not_to permit(FactoryBot.create(:user), product)
    end

    it 'denies access if developer not owner' do
      expect(subject).not_to permit(FactoryBot.create(:user, position: 'developer'), product)
    end

    it 'grants access if developer and owner' do
      expect(subject).to permit(developer, product)
    end

    it 'grants access if admin' do
      expect(subject).to permit(FactoryBot.create(:user, roles: ['Administrator']), product)
    end
  end

  permissions :edit?, :update? do
    let!(:developer) { FactoryBot.create(:user, position: 'developer') }
    let!(:product) { FactoryBot.create(:product, user_id: developer.id, published_at: Time.zone.now - 1.day) }

    it 'denies access if not developer' do
      expect(subject).not_to permit(FactoryBot.create(:user), product)
    end

    it 'denies access if developer not owner' do
      expect(subject).not_to permit(FactoryBot.create(:user, position: 'developer'), product)
    end

    it 'denies access if developer and owner but published' do
      expect(subject).not_to permit(developer, product)
    end

    it 'grants access if developer and owner and not published' do
      product.update(published_at: nil)
      expect(subject).to permit(developer, product)
    end

    it 'grants access if admin' do
      expect(subject).to permit(FactoryBot.create(:user, roles: ['Administrator']), product)
    end
  end

  permissions :unpublish?, :ask_to_publish? do
    let!(:developer) { FactoryBot.create(:user, position: 'developer') }
    let!(:product) { FactoryBot.create(:product, user_id: developer.id) }

    it 'denies access if not developer' do
      expect(subject).not_to permit(FactoryBot.create(:user), product)
    end

    it 'denies access if developer not owner' do
      expect(subject).not_to permit(FactoryBot.create(:user, position: 'developer'), product)
    end

    it 'grants access if developer and owner' do
      expect(subject).to permit(developer, product)
    end

    it 'grants access if admin' do
      expect(subject).to permit(FactoryBot.create(:user, roles: ['Administrator']), product)
    end
  end

  permissions :index?, :more?, :sort?, :admin_edit?, :admin_update?, :publish?, :promote?, :unpromote? do
    let!(:developer) { FactoryBot.create(:user, position: 'developer') }
    let!(:product) { FactoryBot.create(:product, user_id: developer.id) }

    it 'denies access if not admin' do
      expect(subject).not_to permit(developer, product)
    end

    it 'grants access if admin' do
      expect(subject).to permit(FactoryBot.create(:user, roles: ['Administrator']), product)
    end
  end
end
